@font-face {
  font-family: 'Pipeline';
  src: url('../fonts/pipeline.eot');
  src: url('../fonts/pipeline.eot?#iefix') format('embedded-opentype');
  src: local('☺'), url('../fonts/pipeline.woff') format('woff'), url('../fonts/pipeline.ttf') format('truetype'), url('../fonts/pipeline.svg#IyfZbseF') format('svg');
  font-weight: normal;
  font-style: normal;
}
.relative-url {
  background-image: url(../images/sprite-buttons.png);
}
.relative-url-querystring {
  background-image: url(../images/sprite-buttons.png?v=1.0#foo=bar);
}
.absolute-url {
  background-image: url(/images/sprite-buttons.png);
}
.absolute-full-url {
  background-image: url(http://localhost/images/sprite-buttons.png);
}
.no-protocol-url {
  background-image: url(//images/sprite-buttons.png);
}
.anchor-tag-url {
  background-image: url(#image-gradient);
}
@font-face{src:url(../fonts/pipeline.eot);src:url(../fonts/pipeline.eot?#iefix) format('embedded-opentype'),url(../fonts/pipeline.woff) format('woff'),url(../fonts/pipeline.ttf) format('truetype');}
